"""
class of Huobi restful api client
"""
from requests import Session
from huobi.rest.endpoint import Endpoint


class HuobiRestClientBaseMeta(type):
    def __new__(mcls, name, bases, attrs):
        cls = super(
            HuobiRestClientBaseMeta,
            mcls).__new__(
            mcls,
            name,
            bases,
            attrs)
        endpoints = {
            attr: obj
            for attr, obj in attrs.items()
            if isinstance(obj, Endpoint)
        }
        for attr, obj in endpoints.items():
            obj.__set_name__(cls, attr)
            obj.__doc__ = obj._generate_docs()
        return cls


class HuobiRestClientBase(object, metaclass=HuobiRestClientBaseMeta):
    """
    Huobi restful api client base
    """

    def __init__(
            self,
            access_key=None,
            secret_key=None,
            base_url='https://api.huobi.pro',
            session_params=None,
            request_params=None
    ):
        self.access_key = access_key
        self.secret_key = secret_key
        self.base_url = base_url
        session_params = session_params or {}
        self.request_params = request_params or {}
        with Session(**session_params) as session:
            self.session = session

    def close(self):
        self.session.close()



"""
Helper functions/values for huobi rest client
"""


REQUIRED_HEADERS = {
    'user-agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64) '
                  'AppleWebKit/537.36 (KHTML, like Gecko) '
                  'Chrome/39.0.2171.71 Safari/537.36',
    'content-type': 'application/x-www-form-urlencoded',
}

REQUIRED_POST_HEADERS = {
    'content-type': 'application/json',
    'accept': 'application/json'
}



"""
class of Huobi restful api client, common endpoints
"""
from huobi.rest.endpoints import HuobiRestClientBase
from huobi.rest.endpoint import Endpoint


class HuobiRestClientCommon(HuobiRestClientBase):
    """
    Huobi restful api client
    """

    symbols = Endpoint(
        method='GET',
        path='/v1/common/symbols',
        auth_required=False,
    )

    currencys = currencies = Endpoint(
        method='GET',
        path='/v1/common/currencys'
    )

    timestamp = Endpoint(
        method='GET',
        path='/v1/common/timestamp',
        auth_required=False,
    )

    deposit_withdraw = Endpoint(
        method='GET',
        path='/v1/query/deposit-withdraw',
        params={
            'currency': {
                'required': True
            },
            'type': {
                'required': True,
                'choices': ['deposit', 'withdraw']
            },
            'from': {
                'default': 0,
            },
            'size': {
                'default': 100,
            }
        }
    )



"""
class of Huobi restful api client, account related endpoints
"""
from huobi.rest.endpoints import HuobiRestClientBase
from huobi.rest.endpoint import Endpoint


class HuobiRestClientAccounts(HuobiRestClientBase):
    """
    Huobi restful api client
    """

    accounts = Endpoint(
        method='GET',
        path='/v1/account/accounts',
        auth_required=True,
    )

    balance = Endpoint(
        method='GET',
        path='/v1/account/accounts/{account-id}/balance',
        auth_required=True,
        params={
            'account_id': {
                'required': True,
                'url': 'account-id'
            }
        }
    )



"""
Error classes
"""


class HuobiRestError(Exception):
    """
    Huobi rest client exceptions
    """
    pass


class HuobiRestRequestError(HuobiRestError):
    """
    Huobi rest client request exceptions
    """
    pass


class HuobiRestiApiError(HuobiRestError):
    """
    Huobi server returned errors
    """
    pass


class HuobiRestApiDecodeError(HuobiRestError):
    """
    Whatever returned from api is not json
    """
    pass


class HuobiRestArgumentError(HuobiRestError):
    """
    Argument related errors
    """
    pass



"""
class of Huobi restful api client, market related endpoints
"""
from huobi.rest.endpoints import HuobiRestClientBase
from huobi.rest.endpoint import Endpoint


PERIODS = [
    '1min',
    '5min',
    '15min',
    '30min',
    '60min',
    '1day',
    '1mon',
    '1week',
    '1year']


class HuobiRestClientMarket(HuobiRestClientBase):
    """
    Huobi restful api client
    """

    market_history_kline = Endpoint(
        method='get',
        path='/market/history/kline',
        auth_required=False,
        params={
            'symbol': {
                'required': True,
            },
            'period': {
                'default': '5min',
                'choices': [
                    '1min',
                    '5min',
                    '15min',
                    '30min',
                    '60min',
                    '1day',
                    '1mon',
                    '1week',
                    '1year']},
            'size': {
                'default': '1'}})

    market_depth = Endpoint(
        method="get",
        path="/market/depth",
        auth_required=False,
        params={
            'symbol': {
                'required': True
            },
            'type': {
                'required': True,
                'default': 'step0',
                'choices': [f'step{i}' for i in range(0, 6)]
            }
        }
    )

    market_detail_merged = Endpoint(
        method='get',
        path="/market/detail/merged",
        auth_required=False,
        params={
            'symbol': {
                'required': True
            }
        }
    )

    market_history_trade = Endpoint(
        method='get',
        path='/market/history/trade',
        auth_required=False,
        params={
            'symbol': {
                'required': True
            },
            'size': {
                'default': 1,
                'choices': list(range(1, 2001))
            }
        }
    )

    market_detail = Endpoint(
        method='GET',
        path='/market/detail',
        auth_required=False,
        params={
            'symbol': {
                'required': True
            }
        }
    )



import unittest
import os
from huobi.rest.client import HuobiRestClient
from huobi.rest.error import HuobiRestiApiError

from os.path import join, dirname
from dotenv import load_dotenv

dotenv_path = join(dirname(dirname(dirname(dirname(__file__)))), '.env')
load_dotenv(dotenv_path)


class TestOrderEndpoints(unittest.TestCase):

    def setUp(self):
        access_key = os.environ['ACCESS_KEY']
        secret_key = os.environ['SECRET_KEY']
        self.client = HuobiRestClient(
            access_key=access_key, secret_key=secret_key
        )

    def tearDown(self):
        self.client.close()


class TestOrdersE2E(TestOrderEndpoints):

    def test_end_to_end(self):
        # TODO: need mock server to to this integration test
        # place order
        account_id = self.client.accounts().data['data'][0]['id']
        with self.assertRaises(HuobiRestiApiError):
            self.client.place(
                account_id=account_id,
                symbol='btcusdt',
                amount='1',
                price='1',
                type='buy-limit'
            )



import unittest
from huobi.rest.client import HuobiRestClient
from huobi.rest.error import (
    HuobiRestiApiError
)

import os
from os.path import join, dirname
from dotenv import load_dotenv

dotenv_path = join(dirname(dirname(dirname(dirname(__file__)))), '.env')
load_dotenv(dotenv_path)


class TestCommonEndpoint(unittest.TestCase):

    def setUp(self):
        access_key = os.environ['ACCESS_KEY']
        secret_key = os.environ['SECRET_KEY']
        self.client = HuobiRestClient(
            access_key=access_key, secret_key=secret_key)

    def tearDown(self):
        self.client.close()


class TestCommonSymbols(TestCommonEndpoint):

    def test_success(self):
        res = self.client.symbols()
        self.assertEqual(res.res.status_code, 200)
        self.assertIn('data', res.data)
        self.assertIsInstance(res.data['data'], list)

    def test_authentication_fail(self):
        client = HuobiRestClient(
            access_key='1',
            secret_key='2',
        )
        with self.assertRaises(HuobiRestiApiError):
            client.accounts()


class TestCommonCurrencies(TestCommonEndpoint):

    def test_success(self):
        res = self.client.currencies()
        self.assertEqual(res.res.status_code, 200)

    def test_alias(self):
        res = self.client.currencys()
        self.assertEqual(res.res.status_code, 200)


class TestCommonTimestamp(TestCommonEndpoint):

    def test_success(self):
        res = self.client.timestamp()
        self.assertEqual(res.res.status_code, 200)



import unittest
from huobi.rest.client import HuobiRestClient

import os
from os.path import join, dirname
from dotenv import load_dotenv

dotenv_path = join(dirname(dirname(dirname(dirname(__file__)))), '.env')
load_dotenv(dotenv_path)


class TestAccountEndpoint(unittest.TestCase):

    def setUp(self):
        access_key = os.environ['ACCESS_KEY']
        secret_key = os.environ['SECRET_KEY']
        self.client = HuobiRestClient(
            access_key=access_key, secret_key=secret_key)

    def tearDown(self):
        self.client.close()


class TestAccountAccounts(TestAccountEndpoint):

    def test_success(self):
        res = self.client.accounts()
        self.assertEqual(res.res.status_code, 200)


class TestAccountBalance(TestAccountEndpoint):

    def test_success(self):
        account_id = self.client.accounts().data['data'][0]['id']
        res = self.client.balance(account_id=account_id)
        self.assertEqual(res.res.status_code, 200)



import asyncio
import uuid
import json
import gzip
import aiohttp

from functools import partial


def decode_ws_payload(data):
    return json.loads(gzip.decompress(data).decode('utf-8'))


def encode_ws_payload(data):
    return json.dumps(data)


async def subscribe(topics, on_close=None, on_error=None):
    """
    Please refer to Huobi's documentation for available subscribe channels.

    If callback is not a coroutine function, run_in_executor with default
    Executor will be called.

    Example:
    >>> def btc_callback(data):
            print(data)
    >>> async def eth_callback(data):
            print(data)
    >>> task = subscribe({
            'market.btcusdt.kline.1min': {
                'callback': btc_callback
            },
            'market.ethusdt.kline.1min': {
                'callback': eth_callback
            },
        })
    >>> asyncio.get_event_loop().run_until_complete(task)
    """
    loop = asyncio.get_event_loop()
    session = aiohttp.ClientSession()
    async with session.ws_connect('wss://api.huobi.pro/ws') as ws:
        keys = {
            topic: uuid.uuid4().hex
            for topic in topics
        }
        keyed_channels = {
            v: topics[k]
            for k, v in keys.items()
        }
        subscribed_chanels = {}
        for topic, config in topics.items():
            payload = {
                'sub': topic,
                'id': keys[topic]
            }
            await ws.send_str(encode_ws_payload(payload))
        async for msg in ws:
            if msg.type == aiohttp.WSMsgType.BINARY:
                data = decode_ws_payload(msg.data)

                ping = data.get('ping')
                if ping:
                    reply = encode_ws_payload({'pong': ping})
                    await ws.send_str(
                        reply
                    )

                subbed = data.get('subbed')
                if subbed:
                    if data.get('status') == 'ok':
                        subscribed_chanels[subbed] = keyed_channels[data['id']]

                ch = data.get('ch')
                if ch:
                    cb = subscribed_chanels[ch].get('callback', lambda _: None)
                    if asyncio.iscoroutinefunction(cb):
                        await cb(data)
                    else:
                        loop.run_in_executor(None, partial(cb, data))
            elif msg.type == aiohttp.WSMsgType.CLOSED:
                if on_close:
                    return on_close()
                return
            elif msg.type == aiohttp.WSMsgType.ERROR:
                if on_error:
                    return on_error()
                return



"""
class of Huobi restful api client, order related endpoints
"""
import datetime
from huobi.rest.endpoints import HuobiRestClientBase
from huobi.rest.endpoint import Endpoint


def date_formatter(d):
    if isinstance(d, datetime.datetime):
        return d.date().isoformat()
    if isinstance(d, datetime.date):
        return d.isoformat()
    if isinstance(d, str):
        return d


class HuobiRestClientOrder(HuobiRestClientBase):

    place = Endpoint(
        method='POST',
        path='/v1/order/orders/place',
        auth_required=True,
        params={
            'account_id': {
                'required': True,
                'name': 'account-id'
            },
            'amount': {
                'required': True,
            },
            'price': {
                'required': False,
            },
            'source': {
                'required': False,
            },
            'symbol': {
                'required': True
            },
            'type': {
                'required': True,
                'choices': [
                    'buy-market',
                    'sell-market',
                    'buy-limit',
                    'sell-limit',
                ]
            },
        }
    )

    submit_cancel = Endpoint(
        method='POST',
        path='/v1/order/orders/{order-id}/submitcancel',
        auth_required=True,
        params={
            'order_id': {
                'required': True,
                'url': 'order-id'
            }
        }
    )

    batch_cancel = Endpoint(
        method='POST',
        path='/v1/order/orders/batchcancel',
        auth_required=True,
        params={
            'order_ids': {
                'required': True,
                'name': 'order-ids',
                'type': list
            }
        }
    )

    status = Endpoint(
        method='GET',
        path='/v1/order/orders/{order-id}',
        auth_required=True,
        params={
            'order_id': {
                'required': True,
                'url': 'order-id'
            }
        }
    )

    matchresults = Endpoint(
        method='GET',
        path='/v1/order/orders/{order-id}',
        auth_required=True,
        params={
            'order_id': {
                'required': True,
                'url': 'order-id'
            }
        }
    )

    orders = list_orders = Endpoint(
        method='GET',
        path='/v1/order/orders',
        auth_required=True,
        params={
            'symbol': {
                'required': True,
            },
            'types': {
                'required': False,
                'multiple': True,
                'choices': [
                    'buy-market',
                    'sell-market',
                    'buy-limit',
                    'sell-limit',
                ]
            },
            'start_date': {
                'required': False,
                'formatter': date_formatter
            },
            'end_date': {
                'required': False,
                'formatter': date_formatter
            },
            'states': {
                'required': True,
                'choices': [
                    'pre-submitted',
                    'submitted',
                    'partial-filled',
                    'partial-canceled',
                    'filled',
                    'canceled',
                ]
            },
            'from': {
                'required': False,
            },
            'direct': {
                'required': False,
                'choices': [
                    'prev',
                    'next'
                ]
            },
            'size': {
                'required': False,
            }
        }
    )

    list_matchresults = Endpoint(
        method='GET',
        path='/v1/order/matchresults',
        auth_required=True,
        params={
            'symbol': {
                'required': True,
            },
            'types': {
                'required': False,
                'multiple': True,
                'choices': [
                    'buy-market',
                    'sell-market',
                    'buy-limit',
                    'sell-limit',
                    'buy-ioc',
                    'sell-ioc',
                ]
            },
            'start_date': {
                'required': False,
                'formatter': date_formatter
            },
            'end_date': {
                'required': False,
                'formatter': date_formatter
            },
            'from': {
                'required': False,
            },
            'direct': {
                'required': False,
                'choices': [
                    'prev',
                    'next'
                ]
            },
            'size': {
                'required': False,
            }
        }
    )



"""
Market api tests
"""
import unittest
from huobi.rest.client import HuobiRestClient
from huobi.rest.error import (
    HuobiRestiApiError,
    HuobiRestRequestError,
    HuobiRestArgumentError,
)


class TestMarketDataEndpoint(unittest.TestCase):

    def setUp(self):
        self.client = HuobiRestClient()

    def tearDown(self):
        self.client.close()


class TestMarketDataCommon(TestMarketDataEndpoint):

    def test_invalid_url(self):
        client = HuobiRestClient(
            access_key="1",
            secret_key="1",
            base_url='https://api.huobi.jfkdlsajfdklsa')
        with self.assertRaises(HuobiRestRequestError):
            client.market_history_kline(
                symbol="btcusdt",
                period="1day",
                size=2
            )
        client.close()


class TestMarketHistoryKline(TestMarketDataEndpoint):

    def test_success(self):
        res = self.client.market_history_kline(
            symbol="btcusdt",
            period="1day",
            size=2
        )

        self.assertEqual(res.res.status_code, 200)
        self.assertEqual(len(res.data['data']), 2)

    def test_invalid_period(self):
        with self.assertRaises(HuobiRestArgumentError):
            self.client.market_history_kline(
                symbol="btcusdt",
                period="2day",
                size=2
            )

    # def test_invalid_symbol(self):
    #     Huobi will now return a 404 response with html content
    #     if symbol does not exist
    #
    #     with self.assertRaises(HuobiRestApiDecodeError):
    #         self.client.market_history_kline(
    #             symbol="btcusdt2",
    #             period="1day",
    #             size=2
    #         )

    def test_required_argument(self):
        with self.assertRaises(HuobiRestArgumentError):
            self.client.market_history_kline(
                perioid='1day',
                size=1
            )


class TestMarketDetailMerged(TestMarketDataEndpoint):

    def test_success(self):
        res = self.client.market_detail_merged(symbol='btcusdt')
        self.assertEqual(res.res.status_code, 200)
        self.assertIn('tick', res.data)


class TestMarketDepth(TestMarketDataEndpoint):

    def test_success(self):
        res = self.client.market_depth(symbol='btcusdt')
        self.assertEqual(res.res.status_code, 200)
        self.assertIn('tick', res.data)
        self.assertIn('bids', res.data['tick'])


class TestMarketHistoryTrade(TestMarketDataEndpoint):

    def test_success(self):
        res = self.client.market_history_trade(symbol='btcusdt')
        self.assertEqual(res.res.status_code, 200)
        self.assertEqual(len(res.data['data']), 1)

    def test_invalid_size(self):
        with self.assertRaises(HuobiRestArgumentError):
            self.client.market_history_trade(symbol='btcusdt', size=2002)


class TestMarketDetail(TestMarketDataEndpoint):

    def test_success(self):
        res = self.client.market_detail(symbol='btcusdt')
        self.assertEqual(res.res.status_code, 200)
        self.assertIn('tick', res.data)
        self.assertIn('amount', res.data['tick'])



""" Endpoint and HuobiRestResult class
"""
from datetime import datetime
from urllib.parse import urlparse, parse_qsl, urlencode
from inspect import Signature, Parameter

from huobi.rest.helper import REQUIRED_HEADERS, REQUIRED_POST_HEADERS
from huobi.rest.error import (
    HuobiRestError,
    HuobiRestRequestError,
    HuobiRestiApiError,
    HuobiRestArgumentError,
    HuobiRestApiDecodeError
)
from huobi.utils import hmac_sha256_base64


class HuobiRestEndpointResult(object):
    """
    Huobi Rest Endpoint Result
    """

    def __init__(self, req, res, data):
        self.req = req
        self.res = res
        self.data = data


class Endpoint(object):
    """
    Endpoint class
    """

    def __init__(
            self,
            method,
            path,
            params=None,
            auth_required=True,
    ):
        self.method = method
        self.path = path
        self.auth_required = auth_required
        self.params = params or {}
        self.attr_name = None

    def __set_name__(self, _owner, name):
        self.attr_name = name

    def _sign_url(self, instance, method: str, url: str):
        if not self.auth_required:
            return url
        additional_qs = {
            'SignatureMethod': 'HmacSHA256',
            'SignatureVersion': '2',
            'Timestamp': datetime.utcnow().isoformat(timespec='seconds'),
            'AccessKeyId': instance.access_key,
        }
        url_obj = urlparse(url)
        qs_items = parse_qsl(url_obj.query) + list(additional_qs.items())
        qs_string = urlencode(sorted(qs_items))
        msg = (
            f'{method.upper()}\n'
            f'{url_obj.netloc}\n'
            f'{url_obj.path}\n'
            f'{qs_string}'
        ).strip()
        signature = hmac_sha256_base64(instance.secret_key, msg)
        new_qs_dict = {**dict(qs_items), 'Signature': signature}
        new_url = (
            f'{url_obj.scheme}://{url_obj.netloc}'
            f'{url_obj.path}?{urlencode(new_qs_dict)}'
        )
        return new_url

    @staticmethod
    def _generate_param_docs(param_name, param_config):
        choices = param_config.get('choices', [])
        if len(choices) > 20:
            start = ",".join(map(str, choices[:10]))
            end = ",".join(map(str, choices[-10:]))
            choices = f'[{start},...,{end}]'
        lines = [
            f'  * *{param_name}*',
            f'      this keyword argument takes a list of values \n' if param_config.get('multiple') else '',
            f'      :required by Huobi: {param_config.get("required", False)}',
            f"      :default: {param_config['default']}" if 'default' in param_config else '',
            f"      :choices: {choices}" if 'choices' in param_config else '',
        ]
        return '\n'.join([line for line in lines if line])

    def _generate_docs(self):
        param_docs = '\n'.join([
            self._generate_param_docs(k, v)
            for k, v in self.params.items()
        ])
        doc = [
            ":param \**kwargs:\n  See blow\n :Keyword Arguments:"
            if self.params else "This method does not require arguments",
            f'{param_docs}',
        ]
        return '\n'.join(doc)

    @staticmethod
    def _handle_response(_instance, res):
        try:
            if res.status_code >= 500:
                res.raise_for_status()
        except Exception as exc:
            raise HuobiRestRequestError('Request Error') from exc
        try:
            json_data = res.json()
        except Exception as exc:
            raise HuobiRestApiDecodeError('Json decode error') from exc

        if not json_data.get('status') == 'ok':
            error_code = json_data.get('err-code', 'Unknown error code')
            error_msg = json_data.get('err-msg', 'Unknown error message')
            raise HuobiRestiApiError(
                f'{error_code}: {error_msg} \n'
                f'{res.request.method}: {res.request.url}'
            )

        return HuobiRestEndpointResult(res.request, res, json_data)

    def _get_signature(self):
        parameters = []
        for param_name, param_spec in self.params.items():
            param = Parameter(
                name=param_name,
                kind=Parameter.KEYWORD_ONLY,
                default=(param_spec.get('default') or Parameter.empty) if param_spec.get('required') else None,
            )
            parameters.append(param)
        return Signature(parameters=parameters)

    def _with_signature(self, func):
        func.__signature__ = self._get_signature()
        return func

    def __get__(self, instance, owner):

        @self._with_signature
        def _wrapper(**kwargs):
            if self.auth_required and (
                    not instance.secret_key or not instance.access_key):
                raise HuobiRestError(
                    f'Authentication is required for method: {self.attr_name}')
            query_params = {}
            path = self.path
            for param_name, param_spec in self.params.items():
                required = param_spec.get('required', False)
                choices = param_spec.get('choices', None)
                multiple = param_spec.get('multiple', False)
                if required and (
                        'default' not in param_spec
                ) and not kwargs.get(param_name):
                    raise HuobiRestArgumentError(
                        f'{param_name} is required in {self.attr_name}'
                    )
                param_value = kwargs.get(
                    param_name,
                    param_spec.get('default')
                )
                formatter = param_spec.get('formatter')
                if formatter:
                    param_value = formatter(param_value)
                if choices and param_value:
                    if param_value not in choices and not multiple:
                        raise HuobiRestArgumentError(
                            f'{param_value} is not a'
                            f'valid value for {param_name} \n'
                            f'Choices are {choices}')
                    if multiple and not all([part in choices for part in param_value.split(',')]):
                        raise HuobiRestArgumentError(
                            f'{param_value} contains'
                            f'invalid value for {param_name} \n'
                            f'Choices are {choices}')
                required_type = param_spec.get('type')
                if required_type and not isinstance(param_value, required_type):
                    raise HuobiRestArgumentError(
                        f'{param_name} should be of instance of {required_type}'
                        f'but got {type(param_name)}'
                    )
                if param_value is not None:
                    url_replace = param_spec.get('url')
                    if url_replace:
                        path = path.replace(
                            '{' + url_replace + '}', str(param_value)
                        )
                    else:
                        name = param_spec.get('name', param_name)
                        query_params[name] = param_value

            url = f'{instance.base_url}{path}'
            res = None

            if self.method.lower() == 'get':
                url = f'{url}?{urlencode(query_params)}'
                url = self._sign_url(instance, 'GET', url)
                try:
                    res = instance.session.get(
                        url,
                        headers=REQUIRED_HEADERS,
                        **instance.request_params,
                    )
                except Exception as exc:
                    raise HuobiRestRequestError('Request error') from exc
            if self.method.lower() == 'post':
                url = self._sign_url(instance, 'POST', url)
                try:
                    res = instance.session.post(
                        url,
                        json=query_params,
                        headers=REQUIRED_POST_HEADERS,
                        **instance.request_params,
                    )
                except Exception as exc:
                    raise HuobiRestRequestError('Request error') from exc

            return self._handle_response(instance, res)

        _wrapper.__name__ = self.attr_name
        _wrapper.__doc__ = self._generate_docs()

        return _wrapper



"""
Helper functions
"""
from typing import Union
from hashlib import sha256
import hmac
import base64

BYTES_OR_STR = Union[
    str,
    bytes,
    bytearray
]


def hmac_sha256_base64(key: BYTES_OR_STR, msg: BYTES_OR_STR,
                       encoding: str = 'utf-8') -> bytes:
    """
    Generate a base64 encoded signature based on secrete key and message
    """
    if isinstance(key, str):
        key = key.encode(encoding)
    if isinstance(msg, str):
        msg = msg.encode(encoding)

    hmac_obj = hmac.new(
        key=key,
        msg=msg,
        digestmod=sha256
    )
    return base64.b64encode(
        hmac_obj.digest()
    )



import inspect
from urllib.parse import parse_qs, urlparse
from huobi.rest.error import HuobiRestiApiError, HuobiRestRequestError
from .cases import MockedTestCase


class TestEndpoint(MockedTestCase):
    """
    tests for Endpoint
    """

    def test_submit_cancel_order(self):
        # https://github.com/ericls/huobi/issues/1
        self.client.submit_cancel(order_id='1234')
        self.assertIn('/1234/', self.last_url)
        self.client.submit_cancel(order_id='2345')
        self.assertIn('/2345/', self.last_url)

    def test_none_param_value(self):
        # https://github.com/ericls/huobi/issues/4
        # The idea is that if a value to a param if None, instead of include a None value in the request,
        # just don't include it in the request parameters
        self.client.orders(symbol='xrpusdt', states='submitted')
        url = urlparse(self.last_url)
        query = url.query
        qs = parse_qs(query)
        self.assertNotIn('types', qs)
        self.assertNotIn('direct', qs)

    def test_has_signature(self):
        signature = inspect.signature(self.client.market_history_kline)
        assert all(i in signature.parameters for i in ['size', 'period', 'symbol'])

    def test_400(self):
        # 400 series is client side error. Server will likely return helpful error message
        # Do not throw Request Error, throw ApiError instead
        self.mocked_response_status_code = 403
        self.mocked_response = {'status': 'error', 'err-code': 1, 'err-message': 'test'}
        with self.assertRaises(HuobiRestiApiError):
            self.client.market_history_kline(
                symbol="btcusdt",
                period="1day",
                size=2
            )

    def test_500(self):
        self.mocked_response_status_code = 500
        with self.assertRaises(HuobiRestRequestError):
            self.client.market_history_kline(
                symbol="btcusdt",
                period="1day",
                size=2
            )



from setuptools import find_packages, setup

try:
    import pypandoc
    long_description = pypandoc.convert_file('README.md', 'rst', 'md')
except(IOError, ImportError):
    long_description = open('README.md').read()

setup(
    name='huobi',
    version='0.1.12',
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),
    include_package_data=True,
    license='MIT License',
    description='Huobi Python SDK',
    long_description=long_description,
    url='https://github.com/ericls/huobi',
    author='Shen Li',
    author_email='dustet@gmail.com',
    install_requires=[
        "requests>=2.4.2",
        "aiohttp>=3.1.0",
    ],
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3 :: Only',
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    extras_require={
        'socks': ['PySocks>=1.5.6, !=1.5.7'],
        'socks:sys_platform == "win32" and (python_version == "2.7" or python_version == "2.6")': ['win_inet_pton'],
    },
)



from unittest import TestCase
from huobi import HuobiRestClient


class TestProxies(TestCase):
    def setUp(self):
        self.client = HuobiRestClient(request_params={
            'proxies': {
                'https': 'socks5h://127.0.0.1:1080',
            },
        })

    def test_success(self):
        res = self.client.market_history_kline(
            symbol="btcusdt", period="1day", size=2)

        self.assertEqual(res.res.status_code, 200)
        self.assertEqual(len(res.data['data']), 2)



"""
class of Huobi restful api client
"""
from huobi.rest.endpoints.common import HuobiRestClientCommon
from huobi.rest.endpoints.market import HuobiRestClientMarket
from huobi.rest.endpoints.account import HuobiRestClientAccounts
from huobi.rest.endpoints.order import HuobiRestClientOrder
from huobi.rest.endpoints.query import HuobiRestClientQuery


class HuobiRestClient(
    HuobiRestClientMarket,
    HuobiRestClientCommon,
    HuobiRestClientAccounts,
    HuobiRestClientOrder,
    HuobiRestClientQuery,
):
    """
    Huobi restful api client
    """
    pass



import json
from unittest import TestCase, mock
from huobi import HuobiRestClient


class MockedTestCase(TestCase):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.mocked_response = {'status': 'ok', 'data': {}}
        self.mocked_response_status_code = 200

    def setUp(self):
        access_key = 'foo'
        secret_key = 'bar'
        self.client = HuobiRestClient(
            access_key=access_key, secret_key=secret_key
        )
        self.client.session = self._get_mock_session()
        self.history = {}
        self.last_url = ''

    def tearDown(self):
        self.client.close()
    
    def _get_mock_session(self):

        def raise_exception():
            raise Exception()

        def mocked_request_method(url='', **kwargs):
            url = url or kwargs.get('url')
            self.last_url = url
            self.history[url] = kwargs
            response = mock.Mock()
            response.raise_for_status = mock.Mock()
            if self.mocked_response_status_code > 300:
                response.raise_for_status.side_effect = raise_exception
            response.status_code = self.mocked_response_status_code
            response.json = mock.Mock()
            response.json.return_value = self.mocked_response
            return response

        session = mock.Mock()
        session.post = session.get = request_method = mock.Mock()
        request_method.side_effect = mocked_request_method
        return session
